<?php
namespace SupplyTek\Controllers;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use SupplyTek\Utilities\SessionHandler;
use SupplyTek\Utilities\CrudSupplyTek;

/**
 * Class retrives amount used and total amount from unit's budget
 * 
 * @author Danilo D. Vaz
 */
class BudgetController 
{
    protected $crud;
    protected $sessionHandler;

    public function __construct (CrudSupplyTek $crudSupplyTek, SessionHandler $sessionHandler) 
    {
        $this->crud = $crudSupplyTek;
        $this->sessionHandler = $sessionHandler;
    }

    /**
     * Retrives amount available and amount used
     * 
     * Get total amount and amount spent and calculates the available amount 
     * for unit, according to the period.
     * 
     * @param Request $request Object that receives data from the caller
     * @param Response $response Object that sends back data to the caller
     *
     * @return array
     * @throws Exception
     * 
     * @access public
     * @author Danilo D. Vaz
     */
    public function getRequesterBudget (Request $request, Response $response)
    {
        try {
            $filterCriteria = $request->getFilterCriteria();
            $params['OPERATOR_ID'] = $this->sessionHandler->getSessionData('CDOPERADOR');
            
            $filter = $this->crud->processFilter($filterCriteria);
            $params['BRANCH'] = $this->getBranch($filter);
            $params['PERIOD'] = $this->getPeriod($filter);
            
            $totalAmount = $this->crud->executeQuery('requester-budget', $params)[0]['TOTALMES'];
            $amountSpent = $this->crud->executeQuery('amount-spent-month', $params)[0]['TOTALGASTO'];

            $availableAmount = $totalAmount - $amountSpent;
            $requesterBudget = array(array("AVAILABLE" => $availableAmount, "USED" => $amountSpent));
            
            $data = $this->crud->buildData($requesterBudget, $filterCriteria);

            $response->setMetadata($data);
        } catch (\Exception $e) {
            throw new \Exception ($e->getMessage(), 500);
        }
    }
    
    /**
     * Retrives budget by category
     * 
     * Get total budget by category and amount spent by category, and calculates
     * the available amount by category. According to the period and unit.
     * 
     * @param Request $request Object that receives data from the caller
     * @param Response $response Object that sends back data to the caller
     *
     * @return array
     * @throws Exception
     * 
     * @access public
     * @author Danilo D. Vaz
     */
    public function getBudgetByCategory (Request $request, Response $response)
    {
        try {
            $params['OPERATOR_ID'] = $this->sessionHandler->getSessionData('CDOPERADOR');
            $filterCriteria = $request->getFilterCriteria();
            $filter = $this->crud->processFilter($filterCriteria);
            
            $params['BRANCH'] = $this->getBranch($filter);
            $params['PERIOD'] = $this->getPeriod($filter);
            $dbBudgetByCategory = $this->crud->find($request, $params);
            
            $params['FIRST_MONTH'] = $params['PERIOD'];
            $params['LAST_MONTH'] = $params['PERIOD'];
            unset($params['PERIOD']);
            $dbAmountSpentCategory = $this->crud->executeQuery('amount-spent-category', $params);
            
            $budgetByCategory = $this->buildBudgetByCategory($dbBudgetByCategory, $dbAmountSpentCategory);
            
            $data = array(
                "data" => $budgetByCategory,
                "pagination" => $dbBudgetByCategory['pagination']
            );
            
            $response->setMetadata($data);
        } catch (\Exception $e) {
            throw new \Exception ($e->getMessage(), 500);
        }
    }
    
    /**
     * Function that process unit 
     * 
     * Receives the filter sent by frontend and checks the unit. If not informed
     * any unit, returns the session unit.
     * 
     * @param array $filter Array with filter sent by frontend
     *
     * @return string
     * 
     * @access private
     * @author Danilo D. Vaz
     */
    private function getBranch ($filter)
    {
        if (isset($filter['BRANCH'])) {
            return $filter['BRANCH'];
        }
        
        return $this->sessionHandler->getSessionData('CDFILIAL');
    }
    
    /**
     * Function that process period 
     * 
     * Receives the filter sent by frontend and checks the period. If not informed
     * any period, returns the current month. Execute other validations in period.
     * Returns the first day and the last day of the month.
     * 
     * @param array $filter Array with filter sent by frontend
     *
     * @return string
     * 
     * @access private
     * @author Danilo D. Vaz
     */
    private function getPeriod ($filter)
    {
        if (isset($filter['PERIOD'])) {
            $date = $filter['PERIOD'];
        } else {
            $date = $period = date('m/y');
        }
        
        $date = \DateTime::createFromFormat('m/y', $date);
        
        if (!$date) {
            throw new \Exception("Wrong date format.");
        }

        $date->modify('first day of this month');
        $firstDay = $date->format('d/m/Y');
        
        return $firstDay;
    }
    
    /**
     * Function that build and calculates budget by category 
     * 
     * Receives the total budget by category and amount spent by category, and 
     * calculates the amount available by category.
     * 
     * @param array $dbBudgetByCategory Array with total budget by category
     * @param array $dbAmountSpentCategory Array with amount spent by category
     *
     * @return array
     * 
     * @access private
     * @author Danilo D. Vaz
     */
    private function buildBudgetByCategory ($dbBudgetByCategory, $dbAmountSpentCategory)
    {
        $budgetByCategory = array();
            
        foreach ($dbAmountSpentCategory as $dbAmountSpent) {
            $amountSpent[$dbAmountSpent['CDCATEGORIA']] = $dbAmountSpent['TOTAL_GASTO'];
        }
        
        foreach ($dbBudgetByCategory['data'] as $budget) {
            $amount = isset($amountSpent[$budget['CDCATEGORIA']]) ? $amountSpent[$budget['CDCATEGORIA']] : 0;
            
            $item['AVAILABLE'] = $budget['ORCAMENTO_TOTAL_MES'] - $amount;
            $item['CATEGORY'] = $budget['NMCATEGORIA'];
            $item['USED'] = $amount;
            
            array_push($budgetByCategory, $item);
        }
        
        return $budgetByCategory;
    }
    
    /**
     * Retrives amount available and amount used
     * 
     * Get total amount and amount spent and calculates the available amount 
     * all units, according to the period.
     * 
     * @param Request $request Object that receives data from the caller
     * @param Response $response Object that sends back data to the caller
     *
     * @return array
     * @throws Exception
     * 
     * @access public
     * @author Grazielle Ferreira Antunes
     */
    public function getTotalBudget (Request $request, Response $response)
    {
        try {
            $filterCriteria = $request->getFilterCriteria();
            $params['OPERATOR_ID'] = $this->sessionHandler->getSessionData('CDOPERADOR');
            
            $filter = $this->crud->processFilter($filterCriteria);
            $params['BRANCH'] = 'T';
            $params['PERIOD'] = $this->getPeriod($filter);
            
            $totalAmount = $this->crud->executeQuery('requester-budget', $params)[0]['TOTALMES'];
            $amountSpent = $this->crud->executeQuery('amount-spent-month', $params)[0]['TOTALGASTO'];

            $availableAmount = $totalAmount - $amountSpent;
            $totalBudget = array(array("AVAILABLE" => $availableAmount, "USED" => $amountSpent));
            
            $data = $this->crud->buildData($totalBudget, $filterCriteria);
            
            $response->setMetadata($data);
        } catch (\Exception $e) {
            throw new \Exception ($e->getMessage(), 500);
        }
    }
        
}