<?php

namespace SupplyTek\Utilities;

/**
 * Manipulates Data Format
 * 
 * Class that will have reusable functions to format data.
 * 
 * @author Rômulo Alves Lousada
 */
class FormatData
{
    /**
     * Format the address
     * 
     * Creates a string by concatenating the values received, checking beforehand
     * if the varibles are not empty, to determine if it will need a separator or not.
     * 
     * @param string $street String containing the street 
     * @param string $neighborhood String containing the neighborhood 
     * @param string $county String containing the county 
     * @param string $state String containing the state 
     * @param string $postal String containing the postal code 
     *
     * @return string
     * 
     * @access public
     * @author Rômulo Alves Lousada
     */
    public function formatAddress($street = "", $neighborhood = "", $county = "", $state = "", $postal = "")
    {
        $postal = $this->formatPostalCode($postal);
        
        $address = $street;
        $address .= ($address != "" && ($neighborhood != "" || $county != "")) ? " - " : "";
        $address .= $neighborhood;
        $address .= ($neighborhood != "" && $county != "") ? ", " : "";
        $address .= $county;
        $address .= ($address != "" && $state != "") ? " - " : "";
        $address .= $state;
        $address .= ($address != "" && $postal != "") ? ", " : "";
        $address .= $postal;
        
        return $address;
    }
    
    /**
     * Format billing and mandate addresses
     * 
     * Receives an array with data needed to create both addresses.
     * This data is mainly returned from database and with database column names.
     * 
     * @param array $billing Array with data from database to create addresses.  
     *
     * @return array
     * 
     * @access public
     * @author Rômulo Alves Lousada
     */
    public function formatAddressBillingMandate($billing)
    {
        if ($billing["CDTIPOFATUR"] == "003") {
            $billing["POSTAL_CODE"] = $this->formatPostalCode($billing["NRCEPCLIE"]);
            
            $billingAddress = $billing["NRINSJURCLIE"] . " - " . 
                              $billing["NMRAZSOCCLIE"] . " -- " . 
                              $billing["DSENDECLIE"] .  " - " . 
                              $billing["NMBAIRCLIE"] . " - " . 
                              $billing["POSTAL_CODE"]  . " - " . 
                              $billing["NMMUNICLIE"] . " - " . 
                              $billing["SGESTADOCLIE"];

            $mandateAddress = $billingAddress;
        } else {
            $billing["POSTAL_CODE_BILLING"] = $this->formatPostalCode($billing["NRCEPEMPR"]);
            $billing["POSTAL_CODE_MANDATE"] = $this->formatPostalCode($billing["NRCEPIEST"]);
            
            $billingAddress = $billing["NRINSJUREMPR"] . " - " . 
                              $billing["NMEMPRESA"] . " -- " . 
                              $billing["DSENDEEMPR"] . " - " .
                              $billing["NMBAIREMPR"] . " - " . 
                              $billing["POSTAL_CODE_BILLING"] . " - " . 
                              $billing["NMMUNIEMPR"] . " - " . 
                              $billing["SGESTADOEMPR"];

            $mandateAddress = $billing["NRINSJURIEST"] . " - " . 
                              $billing["NMREPRINSEST"] . " -- " . 
                              $billing["DSENDEIEST"] . " - " . 
                              $billing["NMBAIRIEST"] . " - " . 
                              $billing["POSTAL_CODE_MANDATE"] . " - " .
                              $billing["NRCEPIEST"] . " - " . 
                              $billing["NMMUNICIPIOFAT"] . " - " . 
                              $billing["SGESTADOFAT"];
        }
        
        return array (
            "BILLING_ADDRESS" => $billingAddress,
            "MANDATE_ADDRESS" => $mandateAddress
        );
    }
    
    /**
     * Capitalize the words of the string sent.
     * 
     * @param string $word String that will be capitalized.  
     *
     * @return string
     * 
     * @access public
     * @author Rômulo Alves Lousada
     */
    public function capitalizeWords($word)
    {
        return ucwords(strtolower($word));
    }
    
    /**
     * Format the phone number
     * 
     * Receives a string containing a phone number, removes it's special characters
     * and formats it if the second variable is true.
     * 
     * Formatted: "+## (##) ####{#}-####"
     * Unformatted: "############"
     * 
     * @param string $phone String containing the phone number 
     * @param bool $format (optional) Bool that will $ormat the phone using a mask.
     *
     * @return string
     * 
     * @access public
     * @author Rômulo Alves Lousada
     */
    public function formatPhone($phone, $format = false)
    {
        $phone = $this->removeSpecialCharacters($phone);
        
        if ($phone !== null && $phone != "") {
            
            if (substr($phone, 0, 2) == "55") {
                $phone = substr($phone, 2, strlen($phone));
            }
            
            if ($phone[0] == "0") {
                $phone = substr($phone, 1, strlen($phone));
            }
            
            $phone = "55" . $phone;

            if ($this->validatePhoneNumber($phone)) {
                if ($format) {
                    $mask = strlen($phone) == 12 ? "telefone-fixo" : "telefone-celular";
                    $phone = $this->mask($mask, $phone);
                }
                
                return $phone;
            } else {
                return "Telefone Inválido";
            }
        } else {
            return "Sem Telefone Cadastrado";
        }
    }
    
    /**
     * Format the postal code
     * 
     * Received a string containing the postal code and formats it using the
     * CEP format mask: #####-###.
     * 
     * @param string $postal String containing the postal code
     *
     * @return string
     * 
     * @access public
     * @author Rômulo Alves Lousada
     */
    public function formatPostalCode($postal)
    {
        $postal = $this->removeSpecialCharacters($postal);
        
        if ($postal != "") {
            $postal = $this->mask("CEP", $postal);
        }
        
        return $postal;
    }
    
    /**
     * Validate the entire phone number string
     * 
     * Checks if created and unformatted phone string is valid.
     * 
     * @param string $phone String containing the unformatted phone number
     *
     * @return bool
     * 
     * @access private
     * @author Rômulo Alves Lousada
     */
    private function validatePhoneNumber($phone)
    {
        return preg_match("/^\+?([5]{2})-?([1-9]{2})-?([0-9]{8,9})$/i", $phone);
    }
    
    /**
     * Remove special characters
     * 
     * Receives a string and remove any non word or numeric character.
     * 
     * @param string $string String that will have it's special characters removed
     *
     * @return string
     * 
     * @access private
     * @author Rômulo Alves Lousada
     */
    private function removeSpecialCharacters($string)
    {
        return preg_replace("/(\W+)/i", "", $string);
    }
    
    /**
     * Mask a string using the defined format
     * 
     * Receives a string and a mask name that will be used to determine the format
     * that will be used. Replaces the # in the format with the respective values found
     * on the string sent.
     * 
     * @param string $mask Name of the mask that will be used
     * @param string $string String without format that will have it's value inserted into the mask type.
     *
     * @return string
     * 
     * @access public
     * @author Rômulo Alves Lousada
     */
    public function mask($mask, $string)
    {
        $mask = $this->getMaskType($mask);
        
        $string = str_replace(" ", "", $string);
    
        for ($i=0; $i<strlen($string); $i++) {
            $mask[strpos($mask, "#")] = $string[$i];
        }
    
        return $mask;
    }
    
    /**
     * Return the mask format
     * 
     * @param string $mask Name of the mask that will be used
     *
     * @return string
     * 
     * @access private
     * @author Rômulo Alves Lousada
     */
    private function getMaskType($mask)
    {
        switch(strtolower($mask)) {
            case "cpf":
                $mask = "###.###.###-##";
            break;
            
            case "cnpj":
                $mask = "##.###.###/####-##";
            break;
            
            case "cep":
                $mask = "#####-###";
            break;
            
            case "telefone-fixo":
                $mask = "+## (##) ####-####";
            break;
            
            case "telefone-celular":
                $mask = "+## (##) #####-####";
            break;
        }
        
        return $mask;
    }
    
    /**
     * Create a random alphanumeric string.
     * 
     * @param int $length (optional) Int with length of the generated random string.  
     *
     * @return string
     * 
     * @access public
     * @author Rômulo Alves Lousada
     */
    public function generateRandomString($length = 10)
    {
        $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
        $strlen = strlen($characters);
        
        $random = '';
        
        for ($i = 0; $i < $length; $i++) {
            $random .= $characters[rand(0, $strlen - 1)];
        }
     
        return $random;
    }
}